#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const yargs_1 = __importDefault(require("yargs/yargs"));
const path = __importStar(require("path"));
const cdk_dia_1 = require("../src/cdk-dia");
const rendering = __importStar(require("../src/render/index"));
async function initCli() {
    return (0, yargs_1.default)(process.argv.slice(2)).options({
        'cdk-tree-path': { type: 'string', alias: 'tree', default: 'cdk.out/tree.json', describe: 'Path of synthesized CDK cloud assembly' },
        'target-path': { type: 'string', alias: 'target', default: 'diagram.png', describe: 'Target path for rendered PNG' },
        'collapse': { type: 'boolean', default: true, describe: 'Collapse CDK constructs' },
        'collapse-double-clusters': { type: 'boolean', default: true, describe: 'Collapse CDK constructs with one child that is a cluster itself' },
        'stacks': { type: 'array', describe: 'Stacks to include (if not specified all stacks are diagramed)' },
        'rendering': { type: 'string', choices: [cdk_dia_1.Renderers.GRAPHVIZ, cdk_dia_1.Renderers.CYTOSCAPE], default: cdk_dia_1.Renderers.GRAPHVIZ, describe: 'The rendering engine to use' }
    }).version(false).argv;
}
async function generateDiagram(args) {
    const cdkDia = new cdk_dia_1.CdkDia();
    let includedStacks = false;
    if (args.stacks !== undefined) {
        includedStacks = args.stacks.map(it => it.toString());
    }
    const packageBasePath = path.dirname(require.resolve('../../package.json'));
    cdkDia.generateDiagram(args["cdk-tree-path"], args["target-path"], args.collapse, args["collapse-double-clusters"], packageBasePath, includedStacks, args["rendering"])
        .then((output) => output.userOutput())
        .catch(e => {
        console.error(`Failed to generate diagram - ${e}`);
        throw e;
    });
}
function printError(e) {
    if (e instanceof rendering.RenderingError) {
        notifyRenderingError(e);
    }
    else {
        notifyRenderingError(new rendering.RenderingError(`Unexpected error occurred: ${e.message}`));
    }
}
function notifyRenderingError(e) {
    console.log(`${chalk_1.default.red.bold(`Failed to render diagram: ${e.message}`)}`);
    if (e.fixTips.length > 0)
        console.log(`${chalk_1.default.underline(`What could you try?`)}`);
    e.fixTips.forEach(tip => {
        console.log(`\t${tip}`);
    });
}
initCli()
    .then(args => {
    generateDiagram(args)
        .catch(printError);
})
    .catch(printError);
//# sourceMappingURL=cli.js.map